# -*- coding: utf-8 -*-

from .endpoints import *
from .http import SyncHTTPClient, AsyncHTTPClient


class FortniteAPI:
    r"""A number of options can be passed to the :class:`FortniteAPI`.

    Parameters
    -----------
    api_key: :class:`str`
        The API key from Fortnite-API.com\profile to authenticate.
    run_async: Optional[:class:`bool`]
        A boolean whether the API should run in async mode or not.
        Defaults set to ``True``, in which case the api runs sync.

    Attributes
    -----------
    cosmetics: Union[:class:`SyncCosmeticEndpoints`, :class:`AsyncCosmeticEndpoints`]
        The Cosmetic endpoints. The class depends whether the api runs sync or async.
    creator_code: Union[:class:`SyncCreatorCodeEndpoints`, :class:`AsyncCreatorCodeEndpoints`]
         The Creator Code endpoints. The class depends whether the api runs sync or async.
    news: Union[:class:`SyncNewsEndpoints`, :class:`AsyncNewsEndpoints`]
        The News endpoints. The class depends whether the api runs sync or async.
    shop: Union[:class:`SyncShopEndpoints`, :class:`AsyncShopEndpoints`]
        The Shop endpoints. The class depends whether the api runs sync or async.
    """

    def __init__(self, api_key: str = None, run_async: bool = False):
        self.http = SyncHTTPClient() if not run_async else AsyncHTTPClient()
        if api_key:
            self.http.add_header('Authorization', api_key)

        self.aes = SyncAESEndpoints(self) if not run_async else AsyncAESEndpoints(self)
        self.banner = SyncBannerEndpoints(self) if not run_async else AsyncBannerEndpoints(self)
        self.cosmetics = SyncCosmeticsEndpoints(self) if not run_async else AsyncCosmeticsEndpoints(self)
        self.creator_code = SyncCreatorCodeEndpoints(self) if not run_async else AsyncCreatorCodeEndpoints(self)
        self.map = SyncMapEndpoints(self) if not run_async else AsyncMapEndpoints(self)
        self.news = SyncNewsEndpoints(self) if not run_async else AsyncNewsEndpoints(self)
        self.playlist = SyncPlaylistEndpoints(self) if not run_async else AsyncPlaylistEndpoints(self)
        self.shop = SyncShopEndpoints(self) if not run_async else AsyncShopEndpoints(self)
        self.stats = SyncStatsEndpoints(self) if not run_async else AsyncStatsEndpoints(self)



class Map:

    def __init__(self, data):
        self.blank_image = data['images']['blank']
        self.poi_image = data['images']['pois']
        self.pois = [POI(x) for x in data['pois']]
        self.raw_data = data


class POI:

    def __init__(self, data):
        self.id = data['id']
        self.name = data['name']
        self.location = POILocation(data['location'])
        self.raw_data = data


class POILocation:

    def __init__(self, data):
        self.x = data['x']
        self.y = data['y']
        self.z = data['z']
        self.raw_data = data



import json

import aiohttp
import requests

from .errors import ServiceUnavailable, RateLimited, Unauthorized, NotFound, Forbidden

BASE_URL = 'https://fortnite-api.com/'


class SyncHTTPClient:

    def __init__(self):
        self.headers = {}

    def add_header(self, key, val):
        self.headers[key] = val

    def remove_header(self, key):
        return self.headers.pop(key)

    def get(self, endpoint, params=None):
        response = requests.get(BASE_URL + endpoint, params=params, headers=self.headers)
        try:
            data = response.json()
        except json.JSONDecodeError:
            data = {'error': response.text}
        if response.status_code == 200:
            return data
        elif response.status_code == 401:
            raise Unauthorized(data.get('error', 'Error message not provided!'))
        elif response.status_code == 403:
            raise Forbidden(data.get('error', 'Error message not provided!'))
        elif response.status_code == 404:
            raise NotFound(data.get('error', 'Error message not provided!'))
        elif response.status_code == 429:
            raise RateLimited(data.get('error', 'Error message not provided!'))
        elif response.status_code == 503:
            raise ServiceUnavailable(data.get('error', 'Error message not provided!'))
        else:
            raise Exception(data.get('error', 'Error message not provided') + '. Status Code: {0}'
                            .format(str(response.status_code)))


class AsyncHTTPClient:

    def __init__(self):
        self.headers = {}

    def add_header(self, key, val):
        self.headers[key] = val

    def remove_header(self, key):
        return self.headers.pop(key)

    async def get(self, endpoint, params=None):
        async with aiohttp.ClientSession() as session:
            async with session.get(BASE_URL + endpoint, params=params, headers=self.headers) as response:
                try:
                    data = await response.json()
                except aiohttp.ContentTypeError:
                    data = {'error': await response.text()}
                if response.status == 200:
                    return data
                if response.status == 401:
                    raise Unauthorized(data.get('error', 'Error message not provided!'))
                elif response.status == 403:
                    raise Forbidden(data.get('error', 'Error message not provided!'))
                elif response.status == 404:
                    raise NotFound(data.get('error', 'Error message not provided!'))
                elif response.status == 429:
                    raise RateLimited(data.get('error', 'Error message not provided!'))
                elif response.status == 503:
                    raise ServiceUnavailable(data.get('error', 'Error message not provided!'))
                else:
                    raise Exception(data.get('error', 'Error message not provided') + '. Status Code: {0}'
                                    .format(str(response.status)))



from datetime import datetime


class Playlist:

    def __init__(self, data):
        self.id = data['id']
        self.name = data['name']
        self.sub_name = data['subName']
        self.description = data['description']
        self.game_type = data['gameType']
        self.min_players = data['minPlayers']
        self.max_players = data['maxPlayers']
        self.max_teams = data['maxTeams']
        self.max_team_size = data['maxTeamSize']
        self.max_squads = data['maxSquads']
        self.max_squad_size = data['maxSquadSize']

        self.is_default = data['isDefault']
        self.is_tournament = data['isTournament']
        self.is_limited_time_mode = data['isLimitedTimeMode']
        self.is_large_team_game = data['isLargeTeamGame']
        self.accumulate_to_profile_stats = data['accumulateToProfileStats']
        if data.get('images') is not None:
            images = data['images']
            self.showcase_image = images['showcase']
            self.mission_icon = images['missionIcon']
        else:
            self.showcase_image = None
            self.mission_icon = None
        self.gameplay_tags = [gameplay_tag for gameplay_tag in data.get('gameplayTags')] \
            if data.get('gameplayTags') is not None else None
        self.path = data['path']
        try:
            self.added = datetime.strptime(data.get('added'), '%Y-%m-%dT%H:%M:%S%z')
        except (ValueError, TypeError):
            self.added = None
        self.raw_data = data



class Account:
    """Represents a account.

    Attributes
    -----------
    id: :class:`str`
        The id of the user.
    name: :class:`str`
        The display name of the user.
    raw_data: :class:`dict`
        The raw data from request. Can be used for saving and re-creating the class.
    """

    def __init__(self, data):
        self.id = data.get('id')
        self.name = data.get('name')
        self.external_auths = None  # Adding when User lookup feature is enabled
        self.raw_data = data
